﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;
using Microsoft.Kinect;

namespace KinectCam
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        KinectSensor myKinect;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
        }

        unsafe void updateImage(byte[] colorData)
        {
            int noOfPixelBytes = colorData.Length;

            fixed (byte* imageBase = colorData)
            {
                // For the duration of this block the pointer
                // imageBase points to the video image by
                byte* imagePosition = imageBase;
                byte* imageEnd = imageBase + noOfPixelBytes;

                int newValue;

                while (imagePosition != imageEnd)
                {
                    // Process Blue
                    newValue = *imagePosition + blueOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to green
                    imagePosition++;
                    newValue = *imagePosition + greenOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to red
                    imagePosition++;
                    newValue = *imagePosition + redOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // skip past the alpha
                    imagePosition++;
                    imagePosition++;
                }
            }
        }

        unsafe void updateImageClip(byte[] colorData)
        {
            int noOfPixelBytes = colorData.Length;

            fixed (byte* imageBase = colorData)
            {
                // For the duration of this block the pointer
                // imageBase points to the video image by
                byte* imagePosition = imageBase;
                byte* imageEnd = imageBase + noOfPixelBytes;

                int newValue;

                while (imagePosition != imageEnd)
                {
                    // Process Blue
                    newValue = *imagePosition + blueOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to green
                    imagePosition++;
                    newValue = *imagePosition + greenOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to red
                    imagePosition++;
                    newValue = *imagePosition + redOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // skip past the alpha
                    imagePosition++;
                    imagePosition++;
                }
            }
        }


        unsafe void reflectImage(byte[] colorData, int width, int height)
        {
            fixed (byte* imageBase = colorData)
            {
                // Get the base position as an integer pointer
                int* imagePosition = (int*)imageBase;

                // repeat for each row
                for (int row = 0; row < height; row++)
                {
                    // read from the left edge
                    int* fromPos = imagePosition + (row * width);
                    // write to the right edge
                    int* toPos = fromPos + width - 1;
                    while (fromPos < toPos)
                    {
                        *toPos = *fromPos; // copy the pixel
                        fromPos++; // move towards the middle
                        toPos--;   // move back from the right edge
                    }
                }
            }
        }


        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        private int blueOffset = 0;
        private int redOffset = 0;
        private int greenOffset = 0;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                if (clippingTextBox.IsChecked == true)
                    updateImageClip(colorData);
                else
                    updateImage(colorData);

                if (mirrorTextBox.IsChecked == true)
                    reflectImage(colorData,colorFrame.Width, colorFrame.Height);

                if (takePicture)
                {
                    pictureBitmap = BitmapSource.Create(
                        colorFrame.Width, colorFrame.Height, 96, 96, PixelFormats.Bgr32, null,
                        colorData, colorFrame.Width * colorFrame.BytesPerPixel);
                    takePicture = false;
                }

                updateImage(colorData);

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                    kinectVideo.Source = colorImageBitmap;
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );
            }
        }

        #region Picture saving code

        bool takePicture;
        BitmapSource pictureBitmap = null;

        private void saveButton_Click(object sender, RoutedEventArgs e)
        {
            // Set the flag to trigger a snapshot
            takePicture = true;

            // Configure save file dialog box
            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "SnapShot"; // Default file name
            dlg.DefaultExt = ".jpg"; // Default file extension
            dlg.Filter = "Pictures (.jpg)|*.jpg"; // Filter files by extension

            // Process save file dialog box results
            if (dlg.ShowDialog() == true)
            {
                // Save document
                string filename = dlg.FileName;
                using (FileStream stream = new FileStream(filename, FileMode.Create))
                {
                    JpegBitmapEncoder encoder = new JpegBitmapEncoder();
                    encoder.Frames.Add(BitmapFrame.Create(pictureBitmap));
                    encoder.Save(stream);
                }
            }
        }

        #endregion


        private void redSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            redOffset = (int)redSlider.Value;
        }

        private void greenSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            greenOffset = (int)greenSlider.Value;
        }

        private void blueSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            blueOffset = (int)blueSlider.Value;
        }

    }
}
