﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Threading;
using Microsoft.Kinect;

namespace BugSplat
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }


        #region Kinect setup

        KinectSensor myKinect;

        void setupKinect()
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
                return;
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
                return;
            }

            myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
        }

        void shutdownKinect()
        {
            if (myKinect != null)
                myKinect.Stop();
        }

        #endregion

        #region Color Video display

        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );

                kinectVideoImage.Source = colorImageBitmap;
            }
        }

        #endregion

        #region Game drawing

        private void startGameThread()
        {
            Thread game = new Thread(gameLoop);
            game.IsBackground = true;
            game.Start();
        }

        private void gameLoop()
        {
            while (true)
            {
                Dispatcher.Invoke(new Action(() => updateGame()));
                Thread.Sleep(17);
            }
        }

        double bugY = 0;
        double bugYSpeed = 1;

        Random rand = new Random(1);

        private void updateGame()
        {
            Canvas.SetTop(bugImage, bugY);
            bugY = bugY + bugYSpeed;

            if (bugY > gameCanvas.Height)
            {
                int bugX = rand.Next(0, (int)gameCanvas.Width - (int)bugImage.Width);
                Canvas.SetLeft(bugImage, bugX );
                bugY = -bugImage.Height;
            }
        }

        #endregion

        #region Event Handlers

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            setupKinect();
            startGameThread();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            shutdownKinect();
        }

        #endregion

    }
}
