﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Threading;
using Microsoft.Kinect;
using System.Media;

namespace BugSplat
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        #region Kinect setup and shutdown

        KinectSensor myKinect;

        void setupKinect()
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
                return;
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.SkeletonStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
                return;
            }

            myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
            myKinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(myKinect_SkeletonFrameReady);
        }

        void shutdownKinect()
        {
            if (myKinect != null)
                myKinect.Stop();
        }

        #endregion

        #region Skeleton and Mallet Display

        Brush skeletonBrush = new SolidColorBrush(Colors.Red);

        void addLine(Joint j1, Joint j2)
        {
            Line boneLine = new Line();
            boneLine.Stroke = skeletonBrush;
            boneLine.StrokeThickness = 5;

            ColorImagePoint j1P = myKinect.MapSkeletonPointToColor(j1.Position, ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X1 = j1P.X;
            boneLine.Y1 = j1P.Y;

            ColorImagePoint j2P = myKinect.MapSkeletonPointToColor(j2.Position, ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X2 = j2P.X;
            boneLine.Y2 = j2P.Y;

            malletCanvas.Children.Add(boneLine);
        }

        void myKinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            // Remove the old skeleton
            malletCanvas.Children.Clear();
            Brush brush = new SolidColorBrush(Colors.Red);

            Skeleton[] skeletons = null;

            string skeletonMessage = "No Skeleton";
            string qualityMessage = "";

            using (SkeletonFrame frame = e.OpenSkeletonFrame())
            {
                if (frame != null)
                {
                    skeletons = new Skeleton[frame.SkeletonArrayLength];
                    frame.CopySkeletonDataTo(skeletons);
                }
            }

            if (skeletons == null) return;

            foreach (Skeleton skeleton in skeletons)
            {
                if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                {
                    skeletonMessage = "Tracked";
                    Joint headJoint = skeleton.Joints[JointType.Head];
                    Joint hipCenter = skeleton.Joints[JointType.HipCenter];

                    if (skeleton.ClippedEdges == 0)
                    {
                        qualityMessage = "Good Quality";
                    }
                    else
                    {
                        if ((skeleton.ClippedEdges & FrameEdges.Bottom) != 0)
                            qualityMessage += "Move up ";
                        if ((skeleton.ClippedEdges & FrameEdges.Top) != 0)
                            qualityMessage += "Move down ";
                        if ((skeleton.ClippedEdges & FrameEdges.Right) != 0)
                            qualityMessage += "Move left ";
                        if ((skeleton.ClippedEdges & FrameEdges.Left) != 0)
                            qualityMessage += "Move right ";
                    }

                    // Spine
                    addLine(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter]);
                    addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]);

                    // Left leg
                    addLine(skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter]);
                    addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipLeft]);
                    addLine(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft]);
                    addLine(skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft]);
                    addLine(skeleton.Joints[JointType.AnkleLeft], skeleton.Joints[JointType.FootLeft]);

                    // Right leg
                    addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipRight]);
                    addLine(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight]);
                    addLine(skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight]);
                    addLine(skeleton.Joints[JointType.AnkleRight], skeleton.Joints[JointType.FootRight]);

                    // Left arm
                    addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft]);
                    addLine(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft]);
                    addLine(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]);
                    addLine(skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]);

                    // Right arm
                    addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight]);
                    addLine(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight]);
                    addLine(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]);
                    addLine(skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]);

                    updateMallet(skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]);
                }
            }
            StatusTextBlock.Text = skeletonMessage;
            QualityTextBlock.Text = qualityMessage;
        }

        Brush malletHandleBrush = new SolidColorBrush(Colors.Black);
        Brush malletHeadBrush = new SolidColorBrush(Colors.Red);
        float malletHandleLength = 100;
        float malletHeadLength = 50;
        System.Windows.Vector malletPosition;
        float malletHitRadius = 40;
        bool malletValid = false;

        void updateMallet(Joint j1, Joint j2)
        {
            if (j1.TrackingState != JointTrackingState.Tracked ||
                j2.TrackingState != JointTrackingState.Tracked)
                return;

            // Get the start and end positions of the mallet vector

            ColorImagePoint j1P = myKinect.MapSkeletonPointToColor(j1.Position, ColorImageFormat.RgbResolution640x480Fps30);
            ColorImagePoint j2P = myKinect.MapSkeletonPointToColor(j2.Position, ColorImageFormat.RgbResolution640x480Fps30);

            int dX = j2P.X - j1P.X;
            int dY = j2P.Y - j1P.Y;

            System.Windows.Vector malletDirection = new System.Windows.Vector(dX, dY);

            if (malletDirection.Length < 1) return;

            // Convert into a vector of length 1 unit
            malletDirection.Normalize();

            // now set the length of the mallet 
            System.Windows.Vector handleVector = malletDirection * malletHandleLength;

            Line handleLine = new Line();

            handleLine.Stroke = malletHandleBrush;
            handleLine.StrokeThickness = 10;

            handleLine.X1 = j1P.X;
            handleLine.Y1 = j1P.Y;

            handleLine.X2 = j1P.X + handleVector.X;
            handleLine.Y2 = j1P.Y + handleVector.Y;

            malletCanvas.Children.Add(handleLine);

            Line headLine = new Line();

            headLine.Stroke = malletHeadBrush;
            headLine.StrokeThickness = 50;

            System.Windows.Vector headVector = malletDirection * malletHeadLength;

            headLine.X1 = handleLine.X2;
            headLine.Y1 = handleLine.Y2;

            headLine.X2 = handleLine.X2 + headVector.X;
            headLine.Y2 = handleLine.Y2 + headVector.Y;
            malletCanvas.Children.Add(headLine);

            malletPosition = new System.Windows.Vector(j1P.X, j1P.Y);

            malletPosition = malletPosition + (malletDirection * (malletHandleLength + (malletHeadLength / 2)));

            malletValid = true;
        }

        #endregion

        #region Color Image display

        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );

                kinectVideoImage.Source = colorImageBitmap;
            }
        }


        #endregion

        #region Game drawing

        private void startGameThread()
        {
            Thread game = new Thread(gameLoop);
            game.IsBackground = true;
            game.Start();
        }

        private void gameLoop()
        {
            while (true)
            {
                Dispatcher.Invoke(new Action(() => updateGame()));
                Thread.Sleep(17);
            }
        }

        double bugY = 0;
        double bugX = 0;
        double bugYSpeed = 1;

        Random rand = new Random(1);
        int score = 0;

        SoundPlayer scorePlayer;

        private void updateGame()
        {
            Canvas.SetTop(bugImage, bugY);
            bugY = bugY + bugYSpeed;

            // check for collisions

            if (malletValid)
            {
                double bugCenterX = bugX + (bugImage.Width / 2);
                double bugCenterY = bugY + (bugImage.Height / 2);

                System.Windows.Vector hitVector =
                    new System.Windows.Vector(malletPosition.X - bugCenterX, malletPosition.Y - bugCenterY);
                if (hitVector.Length < malletHitRadius)
                {
                    score++;
                    ScoreTextBlock.Text = score.ToString();
                    scorePlayer.Play();
                    bugX = rand.Next(0, (int)gameCanvas.Width - (int)bugImage.Width);
                    Canvas.SetLeft(bugImage, bugX);
                    bugY = -bugImage.Height;
                }
            }


            if (bugY > gameCanvas.Height)
            {
                bugX = rand.Next(0, (int)gameCanvas.Width - (int)bugImage.Width);
                Canvas.SetLeft(bugImage, bugX);
                bugY = -bugImage.Height;
            }
        }

        #endregion

        #region Event Handlers

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            setupKinect();
            startGameThread();
            scorePlayer = new SoundPlayer("sounds/ding.wav");
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            shutdownKinect();
        }

        #endregion

    }
}
