﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;
using System.IO;
using System.Media;


namespace KinectDepthCam
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        KinectSensor myKinect;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.DepthStream.Enable();
                myKinect.SkeletonStream.Enable();
                myKinect.ColorStream.Enable();

                myKinect.Start();

                myKinect.DepthFrameReady +=new EventHandler<DepthImageFrameReadyEventArgs>(myKinect_DepthFrameReady);

                myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Depth Viewer");
                Application.Current.Shutdown();
            }
        }

        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );

                kinectVideoImage.Source = colorImageBitmap;
            }
        }

        short[] depthData = null;

        byte[] depthColorImage = null;

        WriteableBitmap depthImageBitmap = null;

        void myKinect_DepthFrameReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            using (DepthImageFrame depthFrame = e.OpenDepthImageFrame())
            {
                if (depthFrame == null) return;

                if (depthData == null)
                    depthData = new short[depthFrame.PixelDataLength];

                if (depthColorImage == null)
                    depthColorImage = new byte[depthFrame.PixelDataLength * 4];

                depthFrame.CopyPixelDataTo(depthData);

                int depthColorImagePos = 0;

                for (int depthPos = 0; depthPos < depthFrame.PixelDataLength; depthPos++)
                {
                    int depthValue = depthData[depthPos] >> DepthImageFrame.PlayerIndexBitmaskWidth;
                    int playerNo = depthData[depthPos] & DepthImageFrame.PlayerIndexBitmask;

                    // Check for the invalid values

                    if (depthValue == myKinect.DepthStream.UnknownDepth ||
                        depthValue == myKinect.DepthStream.TooFarDepth ||
                        depthValue == myKinect.DepthStream.TooNearDepth)
                    {
                        // show invalid values as black
                        depthColorImage[depthColorImagePos++] = 0; // Blue
                        depthColorImage[depthColorImagePos++] = 0; // Green
                        depthColorImage[depthColorImagePos++] = 0; // Red
                    }
                    else
                    {
                        byte depthByte = (byte)(255 - (depthValue >> 4));
                        if (playerNo != 0)
                        {
                            // show player values as blue
                            depthColorImage[depthColorImagePos++] = depthByte; // Blue
                            depthColorImage[depthColorImagePos++] = 0; // Green
                            depthColorImage[depthColorImagePos++] = 0; // Red
                        }
                        else
                        {
                            depthColorImage[depthColorImagePos++] = depthByte; // Blue
                            depthColorImage[depthColorImagePos++] = depthByte; // Green
                            depthColorImage[depthColorImagePos++] = depthByte; // Red
                        }
                    }
                    // transparency
                    depthColorImagePos++;
                }

                // we now have a new array of color data

                if (depthImageBitmap == null)
                {
                    this.depthImageBitmap = new WriteableBitmap(
                        depthFrame.Width,
                        depthFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                    kinectDepthImage.Source = depthImageBitmap;
                }

                this.depthImageBitmap.WritePixels(
                    new Int32Rect(0, 0, depthFrame.Width, depthFrame.Height),
                    depthColorImage, // video data
                    depthFrame.Width * 4, // stride,
                    0   // offset into the array - start at 0
                    );
            }
        }
    }
}
