﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Media;
using Microsoft.Kinect;
using KinectUtils;

namespace KinectTwoPlayer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        
        #region Kinect setup

        KinectManager myKinectMgr;
        KinectSensor myKinect;

        void setupKinect()
        {
            try
            {
                myKinectMgr = new KinectManager(
                    0, // Sensor number 0
                    ColorImageFormat.Undefined,                 // No color image needed
                    DepthImageFormat.Resolution320x240Fps30,    // Depth image needed
                    true);                                      // Skeleton image needed

                myKinect = myKinectMgr.Kinect;

                myKinectMgr.DoProcessSkeletons +=
                    new KinectManager.ProcessSkeletons(processSkeletonFrame);

                myKinectMgr.DoStatusChanged +=
                    new KinectManager.StatusChanged(updateStatus);

                myKinectMgr.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Bugsplat Game");
                Application.Current.Shutdown();
            }
        }

        void shutdownKinect()
        {
            if (myKinectMgr != null)
                myKinectMgr.Stop();
        }

        #endregion


        #region Kinect Skeleton Display

        Skeleton[] skeletons = null;

        void processSkeletonFrame(SkeletonFrame frame)
        {
            skeletons = new Skeleton[frame.SkeletonArrayLength];
            frame.CopySkeletonDataTo(skeletons);
            Dispatcher.Invoke(new Action(() => drawSkeletons()));
        }

        Color[] skeletonColors = new[] {
            Colors.Red, Colors.Green, Colors.Blue, Colors.Yellow,Colors.Purple, Colors.Cyan};
        
        void drawSkeletons()
        {
            if (skeletons == null) return;

            skeletonCanvas.Children.Clear();

            for (int i = 0; i < 6; i++)
            {
                Skeleton skeleton = skeletons[i];

                if (skeleton.TrackingState != SkeletonTrackingState.Tracked)
                    continue;
                skeletonBrush = new SolidColorBrush(skeletonColors[i]);
                drawSkeleton(skeleton);
            }
        }

        Brush skeletonBrush = new SolidColorBrush(Colors.Red);

        void addLine(Joint j1, Joint j2)
        {
            Line boneLine = new Line();
            boneLine.Stroke = skeletonBrush;
            boneLine.StrokeThickness = 5;

            ColorImagePoint j1P = myKinect.MapSkeletonPointToColor(j1.Position, ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X1 = j1P.X;
            boneLine.Y1 = j1P.Y;

            ColorImagePoint j2P = myKinect.MapSkeletonPointToColor(j2.Position, ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X2 = j2P.X;
            boneLine.Y2 = j2P.Y;

            skeletonCanvas.Children.Add(boneLine);
        }

        void drawSkeleton(Skeleton skeleton)
        {
            // Spine
            addLine(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter]);
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]);

            // Left leg
            addLine(skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter]);
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipLeft]);
            addLine(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft]);
            addLine(skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft]);
            addLine(skeleton.Joints[JointType.AnkleLeft], skeleton.Joints[JointType.FootLeft]);

            // Right leg
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipRight]);
            addLine(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight]);
            addLine(skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight]);
            addLine(skeleton.Joints[JointType.AnkleRight], skeleton.Joints[JointType.FootRight]);

            // Left arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft]);
            addLine(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft]);
            addLine(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]);
            addLine(skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]);

            // Right arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight]);
            addLine(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight]);
            addLine(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]);
            addLine(skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]);

        }

        #endregion

        #region Status Display

        string newStatus;

        void updateStatus(string status)
        {
            newStatus = status;
            Dispatcher.Invoke(new Action(() => updateMessage()));
        }

        void updateMessage()
        {
            KinectStatusTextBlock.Text = newStatus;
        }

        #endregion

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            setupKinect();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            shutdownKinect();
        }
    }
}
